@@ -3,7 +3,7 @@
    Parser for dhclient config and lease files... */
 
 /*
- * Copyright (c) 2004-2009 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2014 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -24,12 +24,6 @@
  *   <info@isc.org>
  *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
 #include "dhcpd.h"
@@ -59,10 +53,18 @@ isc_result_t read_client_conf ()
 {
 	struct client_config *config;
 	struct interface_info *ip;
-	struct parse *parse;
 	isc_result_t status;
 	unsigned code;
 
+        /* 
+         * TODO: LATER constant is very undescriptive. We should review it and
+         * change it to something more descriptive or even better remove it
+         * completely as it is currently not used.
+         */
+#ifdef LATER
+        struct parse *parse = NULL;
+#endif
+
 	/* Initialize the default request list. */
 	memset(default_requested_options, 0, sizeof(default_requested_options));
 
@@ -128,6 +130,16 @@ isc_result_t read_client_conf ()
 	top_level_config.retry_interval = 300;
 	top_level_config.backoff_cutoff = 15;
 	top_level_config.initial_interval = 3;
+
+	/*
+	 * RFC 2131, section 4.4.1 specifies that the client SHOULD wait a
+	 * random time between 1 and 10 seconds. However, we choose to not
+	 * implement this default. If user is inclined to really have that
+	 * delay, he is welcome to do so, using 'initial-delay X;' parameter
+	 * in config file.
+	 */
+	top_level_config.initial_delay = 0;
+
 	top_level_config.bootp_policy = P_ACCEPT;
 	top_level_config.script_name = path_dhclient_script;
 	top_level_config.requested_options = default_requested_options;
@@ -149,7 +161,6 @@ isc_result_t read_client_conf ()
 					(struct interface_info *)0,
 					&top_level_config);
 
-	parse = NULL;
 	if (status != ISC_R_SUCCESS) {
 		;
 #ifdef LATER
@@ -209,7 +220,7 @@ int read_client_conf_file (const char *name, struct interface_info *ip,
 	const char *val;
 	int token;
 	isc_result_t status;
-	
+
 	if ((file = open (name, O_RDONLY)) < 0)
 		return uerr2isc (errno);
 
@@ -224,7 +235,7 @@ int read_client_conf_file (const char *name, struct interface_info *ip,
 			break;
 		parse_client_statement (cfile, ip, client);
 	} while (1);
-	token = next_token (&val, (unsigned *)0, cfile);
+	skip_token(&val, (unsigned *)0, cfile);
 	status = (cfile -> warnings_occurred
 		  ? DHCP_R_BADPARSE
 		  : ISC_R_SUCCESS);
@@ -324,7 +335,7 @@ void parse_client_statement (cfile, ip, config)
 
 	switch (peek_token (&val, (unsigned *)0, cfile)) {
 	      case INCLUDE:
-		next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		token = next_token (&val, (unsigned *)0, cfile);
 		if (token != STRING) {
 			parse_warn (cfile, "filename string expected.");
@@ -338,7 +349,7 @@ void parse_client_statement (cfile, ip, config)
 		return;
 		
 	      case KEY:
-		next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		if (ip) {
 			/* This may seem arbitrary, but there's a reason for
 			   doing it: the authentication key database is not
@@ -361,7 +372,7 @@ void parse_client_statement (cfile, ip, config)
 
 	      case TOKEN_ALSO:
 		/* consume ALSO */
-		next_token(&val, NULL, cfile);
+		skip_token(&val, NULL, cfile);
 
 		/* consume type of ALSO list. */
 		token = next_token(&val, NULL, cfile);
@@ -429,7 +440,7 @@ void parse_client_statement (cfile, ip, config)
 		/* REQUIRE can either start a policy statement or a
 		   comma-separated list of names of required options. */
 	      case REQUIRE:
-		next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		token = peek_token (&val, (unsigned *)0, cfile);
 		if (token == AUTHENTICATION) {
 			policy = P_REQUIRE;
@@ -439,22 +450,22 @@ void parse_client_statement (cfile, ip, config)
 		return;
 
 	      case IGNORE:
-		next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		policy = P_IGNORE;
 		goto do_policy;
 
 	      case ACCEPT:
-		next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		policy = P_ACCEPT;
 		goto do_policy;
 
 	      case PREFER:
-		next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		policy = P_PREFER;
 		goto do_policy;
 
 	      case DONT:
-		next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		policy = P_DONT;
 		goto do_policy;
 
@@ -487,8 +498,7 @@ void parse_client_statement (cfile, ip, config)
 		break;
 
 	      case OPTION:
-		token = next_token (&val, (unsigned *)0, cfile);
-
+		skip_token(&val, (unsigned *)0, cfile);
 		token = peek_token (&val, (unsigned *)0, cfile);
 		if (token == SPACE) {
 			if (ip) {
@@ -539,12 +549,12 @@ void parse_client_statement (cfile, ip, config)
 		return;
 
 	      case MEDIA:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		parse_string_list (cfile, &config -> media, 1);
 		return;
 
 	      case HARDWARE:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		if (ip) {
 			parse_hardware_param (cfile, &ip -> hw_address);
 		} else {
@@ -555,7 +565,7 @@ void parse_client_statement (cfile, ip, config)
 		return;
 
 	      case ANYCAST_MAC:
-		token = next_token(&val, NULL, cfile);
+		skip_token(&val, NULL, cfile);
 		if (ip != NULL) {
 			parse_hardware_param(cfile, &ip->anycast_mac_addr);
 		} else {
@@ -566,29 +576,29 @@ void parse_client_statement (cfile, ip, config)
 		return;
 
 	      case REQUEST:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		if (config -> requested_options == default_requested_options)
 			config -> requested_options = NULL;
 		parse_option_list (cfile, &config -> requested_options);
 		return;
 
 	      case TIMEOUT:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		parse_lease_time (cfile, &config -> timeout);
 		return;
 
 	      case RETRY:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		parse_lease_time (cfile, &config -> retry_interval);
 		return;
 
 	      case SELECT_TIMEOUT:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		parse_lease_time (cfile, &config -> select_interval);
 		return;
 
 	      case OMAPI:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		token = next_token (&val, (unsigned *)0, cfile);
 		if (token != PORT) {
 			parse_warn (cfile,
@@ -614,7 +624,7 @@ void parse_client_statement (cfile, ip, config)
 		return;
 		
 	      case DO_FORWARD_UPDATE:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		token = next_token (&val, (unsigned *)0, cfile);
 		if (!strcasecmp (val, "on") ||
 		    !strcasecmp (val, "true"))
@@ -631,27 +641,32 @@ void parse_client_statement (cfile, ip, config)
 		return;
 
 	      case REBOOT:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		parse_lease_time (cfile, &config -> reboot_timeout);
 		return;
 
 	      case BACKOFF_CUTOFF:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		parse_lease_time (cfile, &config -> backoff_cutoff);
 		return;
 
 	      case INITIAL_INTERVAL:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		parse_lease_time (cfile, &config -> initial_interval);
 		return;
 
+	      case INITIAL_DELAY:
+		skip_token(&val, (unsigned *)0, cfile);
+		parse_lease_time (cfile, &config -> initial_delay);
+		return;
+
 	      case SCRIPT:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		parse_string (cfile, &config -> script_name, (unsigned *)0);
 		return;
 
 	      case VENDOR:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		token = next_token (&val, (unsigned *)0, cfile);
 		if (token != OPTION) {
 			parse_warn (cfile, "expecting 'vendor option space'");
@@ -686,14 +701,14 @@ void parse_client_statement (cfile, ip, config)
 		return;
 
 	      case INTERFACE:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		if (ip)
 			parse_warn (cfile, "nested interface declaration.");
 		parse_interface_declaration (cfile, config, (char *)0);
 		return;
 
 	      case PSEUDO:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		token = next_token (&val, (unsigned *)0, cfile);
 		name = dmalloc (strlen (val) + 1, MDL);
 		if (!name)
@@ -703,17 +718,17 @@ void parse_client_statement (cfile, ip, config)
 		return;
 		
 	      case LEASE:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		parse_client_lease_statement (cfile, 1);
 		return;
 
 	      case ALIAS:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		parse_client_lease_statement (cfile, 2);
 		return;
 
 	      case REJECT:
-		token = next_token (&val, (unsigned *)0, cfile);
+		skip_token(&val, (unsigned *)0, cfile);
 		parse_reject_statement (cfile, config);
 		return;
 
@@ -784,7 +799,7 @@ parse_option_list(struct parse *cfile, struct option ***list)
 		}
 		if (!is_identifier (token)) {
 			parse_warn (cfile, "%s: expected option name.", val);
-			token = next_token (&val, (unsigned *)0, cfile);
+			skip_token(&val, (unsigned *)0, cfile);
 			skip_to_semi (cfile);
 			return 0;
 		}
@@ -900,7 +915,7 @@ void parse_interface_declaration (cfile, outer_config, name)
 			break;
 		parse_client_statement (cfile, ip, client -> config);
 	} while (1);
-	token = next_token (&val, (unsigned *)0, cfile);
+	skip_token(&val, (unsigned *)0, cfile);
 }
 
 int interface_or_dummy (struct interface_info **pi, const char *name)
@@ -1027,7 +1042,7 @@ void parse_client_lease_statement (cfile, is_static)
 			break;
 		parse_client_lease_declaration (cfile, lease, &ip, &client);
 	} while (1);
-	token = next_token (&val, (unsigned *)0, cfile);
+	skip_token(&val, (unsigned *)0, cfile);
 
 	/* If the lease declaration didn't include an interface
 	   declaration that we recognized, it's of no use to us. */
@@ -1521,12 +1536,12 @@ parse_client6_lease_statement(struct parse *cfile)
 static struct dhc6_ia *
 parse_client6_ia_na_statement(struct parse *cfile)
 {
-	struct data_string id;
 	struct option_cache *oc = NULL;
 	struct dhc6_ia *ia;
 	struct dhc6_addr **addr;
 	const char *val;
-	int token, no_semi;
+	int token, no_semi, len;
+	u_int8_t buf[5];
 
 	ia = dmalloc(sizeof(*ia), MDL);
 	if (ia == NULL) {
@@ -1537,20 +1552,11 @@ parse_client6_ia_na_statement(struct parse *cfile)
 	ia->ia_type = D6O_IA_NA;
 
 	/* Get IAID. */
-	memset(&id, 0, sizeof(id));
-	if (parse_cshl(&id, cfile)) {
-		if (id.len == 4)
-			memcpy(ia->iaid, id.data, 4);
-		else {
-			parse_warn(cfile, "Expecting IAID of length 4, got %d.",
-				   id.len);
-			skip_to_semi(cfile);
-			dfree(ia, MDL);
-			return NULL;
-		}
-		data_string_forget(&id, MDL);
+	len = parse_X(cfile, buf, 5);
+	if (len == 4) {
+		memcpy(ia->iaid, buf, 4);
 	} else {
-		parse_warn(cfile, "Expecting IAID.");
+		parse_warn(cfile, "Expecting IAID of length 4, got %d.", len);
 		skip_to_semi(cfile);
 		dfree(ia, MDL);
 		return NULL;
@@ -1658,12 +1664,12 @@ parse_client6_ia_na_statement(struct parse *cfile)
 static struct dhc6_ia *
 parse_client6_ia_ta_statement(struct parse *cfile)
 {
-	struct data_string id;
 	struct option_cache *oc = NULL;
 	struct dhc6_ia *ia;
 	struct dhc6_addr **addr;
 	const char *val;
-	int token, no_semi;
+	int token, no_semi, len;
+	u_int8_t buf[5];
 
 	ia = dmalloc(sizeof(*ia), MDL);
 	if (ia == NULL) {
@@ -1674,20 +1680,11 @@ parse_client6_ia_ta_statement(struct parse *cfile)
 	ia->ia_type = D6O_IA_TA;
 
 	/* Get IAID. */
-	memset(&id, 0, sizeof(id));
-	if (parse_cshl(&id, cfile)) {
-		if (id.len == 4)
-			memcpy(ia->iaid, id.data, 4);
-		else {
-			parse_warn(cfile, "Expecting IAID of length 4, got %d.",
-				   id.len);
-			skip_to_semi(cfile);
-			dfree(ia, MDL);
-			return NULL;
-		}
-		data_string_forget(&id, MDL);
+	len = parse_X(cfile, buf, 5);
+	if (len == 4) {
+		memcpy(ia->iaid, buf, 4);
 	} else {
-		parse_warn(cfile, "Expecting IAID.");
+		parse_warn(cfile, "Expecting IAID of length 4, got %d.", len);
 		skip_to_semi(cfile);
 		dfree(ia, MDL);
 		return NULL;
@@ -1775,12 +1772,12 @@ parse_client6_ia_ta_statement(struct parse *cfile)
 static struct dhc6_ia *
 parse_client6_ia_pd_statement(struct parse *cfile)
 {
-	struct data_string id;
 	struct option_cache *oc = NULL;
 	struct dhc6_ia *ia;
 	struct dhc6_addr **pref;
 	const char *val;
-	int token, no_semi;
+	int token, no_semi, len;
+	u_int8_t buf[5];
 
 	ia = dmalloc(sizeof(*ia), MDL);
 	if (ia == NULL) {
@@ -1791,20 +1788,11 @@ parse_client6_ia_pd_statement(struct parse *cfile)
 	ia->ia_type = D6O_IA_PD;
 
 	/* Get IAID. */
-	memset(&id, 0, sizeof(id));
-	if (parse_cshl(&id, cfile)) {
-		if (id.len == 4)
-			memcpy(ia->iaid, id.data, 4);
-		else {
-			parse_warn(cfile, "Expecting IAID of length 4, got %d.",
-				   id.len);
-			skip_to_semi(cfile);
-			dfree(ia, MDL);
-			return NULL;
-		}
-		data_string_forget(&id, MDL);
+	len = parse_X(cfile, buf, 5);
+	if (len == 4) {
+		memcpy(ia->iaid, buf, 4);
 	} else {
-		parse_warn(cfile, "Expecting IAID.");
+		parse_warn(cfile, "Expecting IAID of length 4, got %d.", len);
 		skip_to_semi(cfile);
 		dfree(ia, MDL);
 		return NULL;
@@ -2245,4 +2233,3 @@ int parse_allow_deny (oc, cfile, flag)
 	skip_to_semi (cfile);
 	return 0;
 }
-
