@@ -3,7 +3,8 @@
    Find and identify the network interfaces. */
 
 /*
- * Copyright (c) 2004-2009 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2013-2014 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2009,2011 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -24,16 +25,13 @@
  *   <info@isc.org>
  *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
 #include "dhcpd.h"
 
+/* length of line we can read from the IF file, 256 is too small in some cases */
+#define IF_LINE_LENGTH 1024
+
 #define BSD_COMP		/* needed on Solaris for SIOCGLIFNUM */
 #include <sys/ioctl.h>
 #include <errno.h>
@@ -57,14 +55,10 @@ struct in_addr limited_broadcast;
 int local_family = AF_INET;
 struct in_addr local_address;
 
-#ifdef DHCPv6
-struct in6_addr local_address6;
-#endif /* DHCPv6 */
-
-void (*bootp_packet_handler) PROTO ((struct interface_info *,
-				     struct dhcp_packet *, unsigned,
-				     unsigned int,
-				     struct iaddr, struct hardware *));
+void (*bootp_packet_handler) (struct interface_info *,
+			      struct dhcp_packet *, unsigned,
+			      unsigned int,
+			      struct iaddr, struct hardware *);
 
 #ifdef DHCPv6
 void (*dhcpv6_packet_handler)(struct interface_info *,
@@ -309,12 +303,15 @@ int
 next_iface(struct iface_info *info, int *err, struct iface_conf_list *ifaces) {
 	struct LIFREQ *p;
 	struct LIFREQ tmp;
+	isc_boolean_t foundif;
 #if defined(sun) || defined(__linux)
 	/* Pointer used to remove interface aliases. */
 	char *s;
 #endif
 
 	do {
+		foundif = ISC_FALSE;
+
 		if (ifaces->next >= ifaces->num) {
 			*err = 0;
 			return 0;
@@ -328,6 +325,13 @@ next_iface(struct iface_info *info, int *err, struct iface_conf_list *ifaces) {
 			log_error("Interface name '%s' too long", p->lifr_name);
 			return 0;
 		}
+
+		/* Reject if interface address family does not match */
+		if (p->lifr_addr.ss_family != local_family) {
+			ifaces->next++;
+			continue;
+		}
+
 		strcpy(info->name, p->lifr_name);
 		memset(&info->addr, 0, sizeof(info->addr));
 		memcpy(&info->addr, &p->lifr_addr, sizeof(p->lifr_addr));
@@ -340,7 +344,9 @@ next_iface(struct iface_info *info, int *err, struct iface_conf_list *ifaces) {
 		}
 #endif /* defined(sun) || defined(__linux) */
 
-	} while (strncmp(info->name, "dummy", 5) == 0);
+		foundif = ISC_TRUE;
+	} while ((foundif == ISC_FALSE) ||
+		 (strncmp(info->name, "dummy", 5) == 0));
 	
 	memset(&tmp, 0, sizeof(tmp));
 	strcpy(tmp.lifr_name, info->name);
@@ -405,7 +411,7 @@ struct iface_info {
  */
 int 
 begin_iface_scan(struct iface_conf_list *ifaces) {
-	char buf[256];
+	char buf[IF_LINE_LENGTH];
 	int len;
 	int i;
 
@@ -478,7 +484,7 @@ begin_iface_scan(struct iface_conf_list *ifaces) {
  */
 static int
 next_iface4(struct iface_info *info, int *err, struct iface_conf_list *ifaces) {
-	char buf[256];
+	char buf[IF_LINE_LENGTH];
 	int len;
 	char *p;
 	char *name;
@@ -602,7 +608,7 @@ next_iface4(struct iface_info *info, int *err, struct iface_conf_list *ifaces) {
  */
 static int
 next_iface6(struct iface_info *info, int *err, struct iface_conf_list *ifaces) {
-	char buf[256];
+	char buf[IF_LINE_LENGTH];
 	int len;
 	char *p;
 	char *name;
@@ -958,7 +964,12 @@ discover_interfaces(int state) {
 		   point-to-point in case an OS incorrectly marks them
 		   as broadcast). Also skip down interfaces unless we're
 		   trying to get a list of configurable interfaces. */
-		if (((!(info.flags & IFF_BROADCAST) ||
+		if ((((local_family == AF_INET &&
+		    !(info.flags & IFF_BROADCAST)) ||
+#ifdef DHCPv6
+		    (local_family == AF_INET6 &&
+		    !(info.flags & IFF_MULTICAST)) ||
+#endif
 		      info.flags & IFF_LOOPBACK ||
 		      info.flags & IFF_POINTOPOINT) && !tmp) ||
 		    (!(info.flags & IFF_UP) &&
@@ -1224,7 +1235,7 @@ discover_interfaces(int state) {
 			    (state == DISCOVER_RELAY)) {
 				if_register6(tmp, 1);
 			} else {
-				if_register6(tmp, 0);
+				if_register_linklocal6(tmp);
 			}
 #endif /* DHCPv6 */
 		}
@@ -1280,13 +1291,14 @@ discover_interfaces(int state) {
 				   tmp -> name, isc_result_totext (status));
 
 #if defined(DHCPv6)
-		/* Only register the first interface for V6, since they all
-		 * use the same socket.  XXX: This has some messy side
-		 * effects if we start dynamically adding and removing
-		 * interfaces, but we're well beyond that point in terms of
-		 * mess.
+		/* Only register the first interface for V6, since
+		 * servers and relays all use the same socket.
+		 * XXX: This has some messy side effects if we start
+		 * dynamically adding and removing interfaces, but
+		 * we're well beyond that point in terms of mess.
 		 */
-		if (local_family == AF_INET6)
+		if (((state == DISCOVER_SERVER) || (state == DISCOVER_RELAY)) &&
+		    (local_family == AF_INET6))
 			break;
 #endif
 	} /* for (tmp = interfaces; ... */
@@ -1386,14 +1398,37 @@ isc_result_t got_one (h)
 	if (result == 0)
 		return ISC_R_UNEXPECTED;
 
-	/* If we didn't at least get the fixed portion of the BOOTP
-	   packet, drop the packet.  We're allowing packets with no
-	   sname or filename, because we're aware of at least one
-	   client that sends such packets, but this definitely falls
-	   into the category of being forgiving. */
-	if (result < DHCP_FIXED_NON_UDP - DHCP_SNAME_LEN - DHCP_FILE_LEN)
+	/*
+	 * If we didn't at least get the fixed portion of the BOOTP
+	 * packet, drop the packet.
+	 * Previously we allowed packets with no sname or filename
+	 * as we were aware of at least one client that did.  But
+	 * a bug caused short packets to not work and nobody has
+	 * complained, it seems rational to tighten up that
+	 * restriction.
+	 */
+	if (result < DHCP_FIXED_NON_UDP)
 		return ISC_R_UNEXPECTED;
 
+#if defined(IP_PKTINFO) && defined(IP_RECVPKTINFO) && defined(USE_V4_PKTINFO)
+	{
+		/* We retrieve the ifindex from the unused hfrom variable */
+		unsigned int ifindex;
+
+		memcpy(&ifindex, hfrom.hbuf, sizeof (ifindex));
+
+		/*
+		 * Seek forward from the first interface to find the matching
+		 * source interface by interface index.
+		 */
+		ip = interfaces;
+		while ((ip != NULL) && (if_nametoindex(ip->name) != ifindex))
+			ip = ip->next;
+		if (ip == NULL)
+			return ISC_R_NOTFOUND;
+	}
+#endif
+
 	if (bootp_packet_handler) {
 		ifrom.len = 4;
 		memcpy (ifrom.iabuf, &from.sin_addr, ifrom.len);
@@ -1451,6 +1486,7 @@ got_one_v6(omapi_object_t *h) {
 		memcpy(ifrom.iabuf, &from.sin6_addr, ifrom.len);
 
 		/* Seek forward to find the matching source interface. */
+		ip = interfaces;
 		while ((ip != NULL) && (if_nametoindex(ip->name) != if_idx))
 			ip = ip->next;
 
