@@ -3,7 +3,8 @@
    Tables of information... */
 
 /*
- * Copyright (c) 2004-2008 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2011-2012 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2009 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,14 +23,8 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   http://www.isc.org/
+ *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
 #include "dhcpd.h"
@@ -63,9 +58,10 @@ HASH_FUNCTIONS (option_code, const unsigned *, struct option,
        some event.  The special all-ones value means 'infinite'.  May either
        be printed as a decimal, eg, "3600", or as this name, eg, "infinite".
    f - flag (true or false)
-   A - array of whatever precedes (e.g., IA means array of IP addresses)
-   a - array of the preceding character (e.g., IIa means two or more IP
-       addresses)
+   A - array of all that precedes (e.g., fIA means array of records of
+       a flag and an IP address)
+   a - array of the preceding character (e.g., fIa means a single flag
+       followed by an array of IP addresses)
    U - name of an option space (universe)
    F - implicit flag - the presence of the option indicates that the
        flag is true.
@@ -92,6 +88,7 @@ HASH_FUNCTIONS (option_code, const unsigned *, struct option,
    d - Domain name (i.e., FOO or FOO.BAR).
    D - Domain list (i.e., example.com eng.example.com)
    c - When following a 'D' atom, enables compression pointers.
+   Z - Zero-length option
 */
 
 struct universe dhcp_universe;
@@ -176,8 +173,11 @@ static struct option dhcp_options[] = {
 	{ "user-class", "t",			&dhcp_universe,  77, 1 },
 	{ "slp-directory-agent", "fIa",		&dhcp_universe,  78, 1 },
 	{ "slp-service-scope", "fto",		&dhcp_universe,  79, 1 },
+	/* 80 is the zero-length rapid-commit (RFC 4039) */
 	{ "fqdn", "Efqdn.",			&dhcp_universe,  81, 1 },
 	{ "relay-agent-information", "Eagent.",	&dhcp_universe,  82, 1 },
+	/* 83 is iSNS (RFC 4174) */
+	/* 84 is unassigned */
 	{ "nds-servers", "IA",			&dhcp_universe,  85, 1 },
 	{ "nds-tree-name", "t",			&dhcp_universe,  86, 1 },
 	{ "nds-context", "t",			&dhcp_universe,  87, 1 },
@@ -188,10 +188,12 @@ static struct option dhcp_options[] = {
 	{ "bcms-controller-names", "D",		&dhcp_universe,  88, 1 },
 	{ "bcms-controller-address", "Ia",	&dhcp_universe,  89, 1 },
 
+	/* 90 is the authentication option (RFC 3118) */
+
 	{ "client-last-transaction-time", "L",  &dhcp_universe,  91, 1 },
 	{ "associated-ip", "Ia",                &dhcp_universe,  92, 1 },
 #if 0
-	/* Not defined by RFC yet */
+	/* Defined by RFC 4578 */
 	{ "pxe-system-type", "S",		&dhcp_universe,  93, 1 },
 	{ "pxe-interface-id", "BBB",		&dhcp_universe,  94, 1 },
 	{ "pxe-client-id", "BX",		&dhcp_universe,  97, 1 },
@@ -205,7 +207,7 @@ static struct option dhcp_options[] = {
 	{ "vivco", "Evendor-class.",		&dhcp_universe, 124, 1 },
 	{ "vivso", "Evendor.",			&dhcp_universe, 125, 1 },
 #if 0
-	/* Not defined by RFC yet.
+	/* Referenced by RFC 4578.
 	 * DO NOT UNCOMMENT THESE DEFINITIONS: these names are placeholders
 	 * and will not be used in future versions of the software.
 	 */
@@ -223,7 +225,7 @@ static struct option dhcp_options[] = {
 	{ "tftp-server-address", "Ia",		&dhcp_universe, 150, 1 },
 #endif
 #if 0
-	/* PXELINUX options: not defined by RFC yet */
+	/* PXELINUX options: defined by RFC 5071 */
 	{ "pxelinux-magic", "BBBB",		&dhcp_universe, 208, 1 },
 	{ "loader-configfile", "t",		&dhcp_universe, 209, 1 },
 	{ "loader-pathprefix", "t",		&dhcp_universe, 210, 1 },
@@ -325,6 +327,11 @@ static struct option dhcpv6_options[] = {
 	{ "relay-msg", "X",			&dhcpv6_universe,  9, 1 },
 
 	/* Option code 10 is curiously unassigned. */
+	/* 
+	 * In draft-ietf-dhc-dhcpv6-25 there were two OPTION_CLIENT_MSG and
+	 * OPTION_SERVER_MSG options. They were eventually unified as
+	 * OPTION_RELAY_MSG, hence no option with value of 10. 
+	 */
 #if 0
 	/* XXX: missing suitable atoms for the auth option.  We may want
 	 * to 'virtually encapsulate' this option a la the fqdn option
@@ -336,7 +343,7 @@ static struct option dhcpv6_options[] = {
 #endif
 	{ "unicast", "6",			&dhcpv6_universe, 12, 1 },
 	{ "status-code", "Nstatus-codes.to",	&dhcpv6_universe, 13, 1 },
-	{ "rapid-commit", "",			&dhcpv6_universe, 14, 1 },
+	{ "rapid-commit", "Z",			&dhcpv6_universe, 14, 1 },
 #if 0
 	/* XXX: user-class contents are of the form "StA" where the
 	 * integer describes the length of the text field.  We don't have
@@ -349,7 +356,7 @@ static struct option dhcpv6_options[] = {
 	{ "vendor-opts", "Evsio.",		&dhcpv6_universe, 17, 1 },
 	{ "interface-id", "X",			&dhcpv6_universe, 18, 1 },
 	{ "reconf-msg", "Ndhcpv6-messages.",	&dhcpv6_universe, 19, 1 },
-	{ "reconf-accept", "",			&dhcpv6_universe, 20, 1 },
+	{ "reconf-accept", "Z",			&dhcpv6_universe, 20, 1 },
 
 				/* RFC3319 OPTIONS */
 
@@ -392,7 +399,7 @@ static struct option dhcpv6_options[] = {
 
 	/* Not yet considering for inclusion. */
 #if 0
-			/* RFC-ietf-geopriv-dhcp-civil-09.txt */
+			/* RFC4776 OPTIONS */
 
 	{ "geoconf-civic", "X",			&dhcpv6_universe, 36, 1 },
 #endif
@@ -424,6 +431,30 @@ static struct option dhcpv6_options[] = {
 	 */
 	{ "fqdn", "Efqdn6-if-you-see-me-its-a-bug-bug-bug.",
 						&dhcpv6_universe, 39, 1 },
+
+	/* Not yet considering for inclusion. */
+#if 0
+			/* draft-ietf-dhc-paa-option-05 */
+	{ "pana-agent", "6A",			&dhcpv6_universe, 40, 1 },
+
+			/* RFC4833 OPTIONS */
+
+	{ "new-posix-timezone", "t",		&dhcpv6_universe, 41, 1 },
+	{ "new-tzdb-timezone", "t",		&dhcpv6_universe, 42, 1 },
+
+			/* RFC4994 OPTIONS */
+
+	{ "ero", "SA",				&dhcpv6_universe, 43, 1 },
+#endif
+
+			/* RFC5007 OPTIONS */
+
+	{ "lq-query", "X",			&dhcpv6_universe, 44, 1 },
+	{ "client-data", "X",			&dhcpv6_universe, 45, 1 },
+	{ "clt-time", "L",			&dhcpv6_universe, 46, 1 },
+	{ "lq-relay-data", "6X",		&dhcpv6_universe, 47, 1 },
+	{ "lq-client-link", "6A",		&dhcpv6_universe, 48, 1 },
+
 	{ NULL, NULL, NULL, 0, 0 }
 };
 
@@ -447,6 +478,11 @@ struct enumeration_value dhcpv6_status_code_values[] = {
 	{ "NoBinding",	  3 }, /* Client record (binding) unavailable.	*/
 	{ "NotOnLink",	  4 }, /* Bad prefix for the link.		*/
 	{ "UseMulticast", 5 }, /* Not just good advice.  It's the law.	*/
+	{ "NoPrefixAvail", 6 }, /* Server has no prefixes to assign.	*/
+	{ "UnknownQueryType", 7 }, /* Query-type unknown/unsupported.	*/
+	{ "MalformedQuery", 8 }, /* Leasequery not valid.		*/
+	{ "NotConfigured", 9 }, /* The target address is not in config.	*/
+	{ "NotAllowed",  10 }, /* Server doesn't allow the leasequery.	*/
 	{ NULL, 0 }
 };
 
@@ -456,6 +492,18 @@ struct enumeration dhcpv6_status_codes = {
 	dhcpv6_status_code_values
 };
 
+struct enumeration_value lq6_query_type_values[] = {
+	{ "query-by-address", 1 },
+	{ "query-by-clientid", 2 },
+	{ NULL, 0 }
+};
+
+struct enumeration lq6_query_types = {
+	NULL,
+	"query-types", 2,
+	lq6_query_type_values
+};
+
 struct enumeration_value dhcpv6_message_values[] = {
 	{ "SOLICIT", 1 },
 	{ "ADVERTISE", 2 },
@@ -470,6 +518,8 @@ struct enumeration_value dhcpv6_message_values[] = {
 	{ "INFORMATION-REQUEST", 11 },
 	{ "RELAY-FORW", 12 },
 	{ "RELAY-REPL", 13 },
+	{ "LEASEQUERY", 14 },
+	{ "LEASEQUERY-REPLY", 15 },
 	{ NULL, 0 }
 };
 
@@ -488,7 +538,9 @@ const char *dhcpv6_type_names[] = {
 	"Reconfigure",
 	"Information-request",
 	"Relay-forward",
-	"Relay-reply"
+	"Relay-reply",
+	"Leasequery",
+	"Leasequery-reply"
 };
 const int dhcpv6_type_name_max =
 	(sizeof(dhcpv6_type_names) / sizeof(dhcpv6_type_names[0]));
@@ -545,7 +597,7 @@ const char *hardware_types [] = {
 	"unknown-29",
 	"unknown-30",
 	"unknown-31",
-	"unknown-32",
+	"infiniband",
 	"unknown-33",
 	"unknown-34",
 	"unknown-35",
@@ -792,14 +844,14 @@ option_reference(struct option **dest, struct option *src,
 	         const char * file, int line)
 {
 	if (!dest || !src)
-	        return ISC_R_INVALIDARG;
+	        return DHCP_R_INVALIDARG;
 
 	if (*dest) {
 #if defined(POINTER_DEBUG)
 	        log_fatal("%s(%d): reference store into non-null pointer!",
 	                  file, line);
 #else
-	        return ISC_R_INVALIDARG;
+	        return DHCP_R_INVALIDARG;
 #endif
 	}
 
@@ -813,13 +865,13 @@ int
 option_dereference(struct option **dest, const char *file, int line)
 {
 	if (!dest)
-	        return ISC_R_INVALIDARG;
+	        return DHCP_R_INVALIDARG;
 
 	if (!*dest) {
 #if defined (POINTER_DEBUG)
 	        log_fatal("%s(%d): dereference of null pointer!", file, line);
 #else
-	        return ISC_R_INVALIDARG;
+	        return DHCP_R_INVALIDARG;
 #endif
 	}
 
@@ -827,7 +879,7 @@ option_dereference(struct option **dest, const char *file, int line)
 #if defined (POINTER_DEBUG)
 	        log_fatal("%s(%d): dereference of <= 0 refcnt!", file, line);
 #else
-	        return ISC_R_INVALIDARG;
+	        return DHCP_R_INVALIDARG;
 #endif
 	}
 
