@@ -3,7 +3,8 @@
    Tokens for config file lexer and parser. */
 
 /*
- * Copyright (c) 2004,2008 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2011-2014 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004,2007-2009 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,14 +23,8 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   http://www.isc.org/
+ *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
 enum dhcp_token {
@@ -46,12 +41,14 @@ enum dhcp_token {
 	TILDE = '~',
 	BANG = '!',
 	PERCENT = '%',
- 	PLUS = '+',
+	PLUS = '+',
 	MINUS = '-',
 	ASTERISK = '*',
 	AMPERSAND = '&',
 	PIPE = '|',
 	CARET = '^',
+	ENDOFLINE = '\n',
+	QUESTIONMARK = '?',
 
 	HOST = 256,
 	FIRST_TOKEN = HOST,
@@ -333,19 +330,36 @@ enum dhcp_token {
 	SUBNET6 = 636,
 	HOST_IDENTIFIER = 637,
 	IA_NA = 638,
-	IAADDR = 639,
-	LEASE6 = 640,
-	PREFERRED_LIFE = 641,
-	MAX_LIFE = 642,
-	DEFAULT_DUID = 643,
-	SERVER_DUID = 644,
-	LLT = 645,
-	EN = 646,
-	LL = 647,
-	RANGE6 = 648,
-	WHITESPACE = 649,
-	TOKEN_ALSO = 650,
-	AFTER = 651
+	IA_TA = 639,
+	IA_PD = 640,
+	IAADDR = 641,
+	IAPREFIX = 642,
+	LEASE6 = 643,
+	PREFERRED_LIFE = 644,
+	MAX_LIFE = 645,
+	DEFAULT_DUID = 646,
+	SERVER_DUID = 647,
+	LLT = 648,
+	EN = 649,
+	LL = 650,
+	RANGE6 = 651,
+	WHITESPACE = 652,
+	TOKEN_ALSO = 653,
+	AFTER = 654,
+	ZEROLEN = 655,
+	TEMPORARY = 656,
+	PREFIX6 = 657,
+	FIXED_PREFIX6 = 658,
+	ANYCAST_MAC = 659,
+	CONFLICT_DONE = 660,
+	AUTO_PARTNER_DOWN = 661,
+	GETHOSTNAME = 662,
+	REWIND = 663,
+	INITIAL_DELAY = 664,
+	GETHOSTBYNAME = 665,
+	PRIMARY6 = 666,
+	SECONDARY6 = 667,
+	TOKEN_INFINIBAND = 668
 };
 
 #define is_identifier(x)	((x) >= FIRST_TOKEN &&	\
