@@ -3,7 +3,8 @@
    Definitions for address trees... */
 
 /*
- * Copyright (c) 2004,2007-2008 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2011,2013,2014 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004,2007-2009 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,14 +23,8 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   http://www.isc.org/
+ *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
 /* A pair of pointers, suitable for making a linked list. */
@@ -115,7 +110,7 @@ struct binding_value {
 		struct data_string data;
 		unsigned long intval;
 		int boolean;
-#if defined (NSUPDATE)
+#if defined (NSUPDATE_OLD)
 		ns_updrec *dns;
 #endif
 		struct fundef *fundef;
@@ -197,7 +192,8 @@ enum expr_op {
 	expr_ucase,
 	expr_lcase,
 	expr_regex_match,
-	expr_iregex_match
+	expr_iregex_match,
+	expr_gethostname
 };
 
 struct expression {
@@ -330,9 +326,9 @@ struct universe {
 			    struct binding_scope **,
 			    struct universe *);
 	u_int32_t (*get_tag) (const unsigned char *);
-	void (*store_tag) PROTO ((unsigned char *, u_int32_t));
+	void (*store_tag) (unsigned char *, u_int32_t);
 	u_int32_t (*get_length) (const unsigned char *);
-	void (*store_length) PROTO ((unsigned char *, u_int32_t));
+	void (*store_length) (unsigned char *, u_int32_t);
 	int tag_size, length_size;
 	unsigned site_code_min, end;
 	option_name_hash_t *name_hash;
